use axum::{extract::Extension, http::Method, Router};
use sqlx::PgPool;
use std::env::{self, current_dir};
use tower_http::{cors::CorsLayer, trace::TraceLayer};
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    // Environment
    tracing::info!("Loading dotenv");
    dotenv::dotenv()?;

    // Logging
    tracing_subscriber::registry()
        .with(tracing_subscriber::EnvFilter::new(
            std::env::var("RUST_LOG").unwrap_or_else(|_| {
                "como_bin=debug,tower_http=debug,axum_extra=debug,hyper=info,mio=info,sqlx=info,async_graphql=debug"
                    .into()
            }),
        ))
        .with(tracing_subscriber::fmt::layer())
        .init();

    // Database
    tracing::info!("Creating pool");
    let db_url = env::var("DATABASE_URL")?;
    let pool = PgPool::connect(&db_url).await?;

    // Database Migrate
    tracing::info!("Migrating db");
    sqlx::migrate!("db/migrations").run(&pool).await?;

    tracing::info!("current path: {}", current_dir()?.to_string_lossy());

    // CORS
    let cors = vec!["http://localhost:3000".parse().unwrap()];

    // Webserver
    tracing::info!("Building router");
    let app = Router::new()
        .layer(TraceLayer::new_for_http())
        .layer(Extension(pool))
        .layer(
            CorsLayer::new()
                .allow_origin(cors)
                .allow_headers([axum::http::header::CONTENT_TYPE])
                .allow_methods([Method::GET, Method::POST, Method::OPTIONS]),
        );

    tracing::info!("Starting webserver");
    axum::Server::bind(&"0.0.0.0:3002".parse().unwrap())
        .serve(app.into_make_service())
        .await
        .unwrap();

    Ok(())
}
